@@ -1,6 +1,6 @@
 package Pcore::App::Selenium;
 
-use Pcore qw[-cli -class];
+use Pcore qw[-class];
 
 extends qw[Pcore::App::Alien];
 
@@ -22,57 +22,45 @@ our $CFG = {
         LINUX_XDISPLAY  => 20,
         USER_EXTENSIONS => q[],
         FF_EXTENSIONS   => {
-            firebug    => 'https://addons.mozilla.org/firefox/downloads/latest/1843/addon-1843-latest.xpi',
-            firexpath  => 'https://addons.mozilla.org/firefox/downloads/latest/11900/addon-11900-latest.xpi',
-            noscript   => 'https://addons.mozilla.org/firefox/downloads/latest/722/addon-722-latest.xpi',
-            flashblock => 'https://addons.mozilla.org/firefox/downloads/latest/433/addon-433-latest.xpi',
-
-            # adblock    => 'https://addons.mozilla.org/firefox/downloads/latest/1865/addon-1865-latest.xpi',
+            firebug   => 'https://addons.mozilla.org/firefox/downloads/latest/1843/addon-1843-latest.xpi',
+            firexpath => 'https://addons.mozilla.org/firefox/downloads/latest/11900/addon-11900-latest.xpi',
+            noscript  => 'https://addons.mozilla.org/firefox/downloads/latest/722/addon-722-latest.xpi',
         }
     }
 };
 
-sub _build_selenium_ver {
-    my $self = shift;
-
-    return $self->cfg->{SELENIUM}->{SELENIUM_VER} // $DIST->{SELENIUM}->{SELENIUM_VER};
+# CLI
+sub cli_name ($self) {
+    return 'selenium';
 }
 
-sub _build_selenium_uri {
-    my $self = shift;
+sub _build_selenium_ver ($self) {
+    return $self->cfg->{SELENIUM}->{SELENIUM_VER} // $PROC->{CFG}->{SELENIUM}->{SELENIUM_VER};
+}
 
+sub _build_selenium_uri ($self) {
     my $major_ver = $self->selenium_ver =~ s/\A(\d+[.]\d+)[.].*/$1/smr;
 
-    return qq[http://selenium-release.storage.googleapis.com/$major_ver/selenium-server-standalone-] . $self->selenium_ver . q[.jar];
+    return qq[https://selenium-release.storage.googleapis.com/$major_ver/selenium-server-standalone-] . $self->selenium_ver . q[.jar];
 }
 
-sub _build_ff_ver {
-    my $self = shift;
-
-    return $self->cfg->{SELENIUM}->{FF_VER} // $DIST->{SELENIUM}->{FF_VER};
+sub _build_ff_ver ($self) {
+    return $self->cfg->{SELENIUM}->{FF_VER} // $PROC->{CFG}->{SELENIUM}->{FF_VER};
 }
 
-sub _build_alien_bin_path {
-    my $self = shift;
-
+sub _build_alien_bin_path ($self) {
     return $self->app_dir . 'selenium-server-standalone-' . $self->selenium_ver . q[.jar];
 }
 
-sub _build_ff_dir {
-    my $self = shift;
-
+sub _build_ff_dir ($self) {
     return $self->app_dir . 'firefox-' . ( $MSWIN ? 'win' : 'nix' ) . q[-] . $self->ff_ver;
 }
 
-sub _build_ff_bin_path {
-    my $self = shift;
-
+sub _build_ff_bin_path ($self) {
     return $self->ff_dir . ( $MSWIN ? '/firefox.exe' : '/firefox-bin' );
 }
 
-sub _build_selenium {
-    my $self = shift;
-
+sub _build_selenium ($self) {
     if ( -f $self->alien_bin_path ) {
         $self->_appx_report_warn( q["] . $self->alien_bin_path . q[" already exists. Remove it manually to rebuild] );
     }
@@ -88,14 +76,12 @@ sub _build_selenium {
     return;
 }
 
-sub _build_ff {
-    my $self = shift;
-
+sub _build_ff ($self) {
     if ( -d $self->ff_dir ) {
         $self->_appx_report_warn( q["] . $self->ff_dir . q[" already exists. Remove it manually to rebuild] );
     }
     else {
-        my $uri = q[http://ftp.mozilla.org/pub/mozilla.org/firefox/releases/] . $self->ff_ver . ( $MSWIN ? q[/win32] : q[/linux-x86_64] ) . q[/en-US/];
+        my $uri = q[https://ftp.mozilla.org/pub/mozilla.org/firefox/releases/] . $self->ff_ver . ( $MSWIN ? q[/win32] : q[/linux-x86_64] ) . q[/en-US/];
         $uri .= $MSWIN ? q[Firefox%20Setup%20] . $self->ff_ver . q[.exe] : q[firefox-] . $self->ff_ver . q[.tar.bz2];
         my $store_to = $MSWIN ? $PROC->{TEMP_DIR} . 'firefox-install.exe' : $PROC->{TEMP_DIR} . 'firefox-install.tar.bz2';
 
@@ -114,6 +100,7 @@ sub _build_ff {
 
         if ( !-e $self->ff_bin_path ) {
             P->file->rmtree( $self->ff_dir );
+
             $self->_appx_report_fatal(q[Firefox deployment error]);
         }
 
@@ -123,32 +110,44 @@ sub _build_ff {
     return;
 }
 
-sub _build_ff_extensions {
-    my $self = shift;
-
+sub _build_ff_extensions ($self) {
     require Archive::Zip;
 
-    for my $ext ( sort keys %{ $self->cfg->{SELENIUM}->{FF_EXTENSIONS} } ) {
+    my $cv = AE::cv;
+
+    for my $ext ( sort keys $self->cfg->{SELENIUM}->{FF_EXTENSIONS}->%* ) {
         $self->_appx_report_info(qq[Downloading: $ext]);
 
-        my $tempfile = P->file->tempfile( binmode => ':raw' );
+        P->ua->request(
+            $self->cfg->{SELENIUM}->{FF_EXTENSIONS}->{$ext},
+            buf_size    => 1,
+            on_progress => 1,
+            blocking    => $cv,
+            on_finish   => sub ($res) {
+                if ( $res->status == 200 ) {
+                    my $zip = Archive::Zip->new;
 
-        P->ua->mirror( $self->cfg->{SELENIUM}->{FF_EXTENSIONS}->{$ext}, $tempfile, blocking => 1, on_progress => 1 );
+                    die q[Error reading .zip archive] unless $zip->readFromFileHandle( $res->body, $res->body->path ) == Archive::Zip::AZ_OK();
 
-        my $zip = Archive::Zip->new;
-        my $res = $zip->readFromFileHandle( $tempfile, $tempfile->filename );
-        die q[Error reading .zip archive] unless $res == Archive::Zip::AZ_OK();
+                    my $id = P->data->from_xml( $zip->contents('install.rdf') =~ s/0$//smr )->{RDF}->{Description}->[0]->{'em:id'}->[0]->{content};
 
-        my $id = P->data->from_xml( $zip->contents('install.rdf') =~ s/0$//smr )->{RDF}->{Description}->{'em:id'};
+                    P->file->copy( $res->body->path, $self->ff_dir . qq[/browser/extensions/$id.xpi] );
+                }
+                else {
+                    die qq[Error downloading FF extension];
+                }
 
-        P->file->copy( $tempfile->filename, $self->ff_dir . qq[/browser/extensions/$id.xpi] );
+                return;
+            }
+        );
     }
 
+    $cv->recv;
+
     return;
 }
 
-sub _cmd {
-    my $self = shift;
+sub _cmd ($self) {
 
     # timeout - Controls how long the client is is allowed to be gone before the session is reclaimed (value in seconds)
     # browserTimeout -Controls how long the browser is allowed to hang (value in seconds)
@@ -171,17 +170,11 @@ sub _cmd {
 }
 
 # APP
-around _build_cfg => sub {
-    my $orig = shift;
-    my $self = shift;
-
+around _build_cfg => sub ( $orig, $self ) {
     return P->hash->merge( $self->$orig, $CFG );
 };
 
-around _create_local_cfg => sub {
-    my $orig = shift;
-    my $self = shift;
-
+around _create_local_cfg => sub ( $orig, $self ) {
     my $local_cfg = {
         SELENIUM => {
             SELENIUM_VER => $self->cfg->{SELENIUM}->{SELENIUM_VER},
@@ -193,10 +186,7 @@ around _create_local_cfg => sub {
     return P->hash->merge( $self->$orig, $local_cfg );
 };
 
-around app_build => sub {
-    my $orig = shift;
-    my $self = shift;
-
+around app_build => sub ( $orig, $self ) {
     $self->$orig;
 
     $self->_build_selenium;
@@ -205,15 +195,11 @@ around app_build => sub {
     return;
 };
 
-sub master_proc {
-    my $self = shift;
-
+sub master_proc ($self) {
     return;
 }
 
-sub alien_proc {
-    my $self = shift;
-
+sub alien_proc ($self) {
     if ($MSWIN) {
         P->sys->system( $self->_cmd() );
 
@@ -227,9 +213,35 @@ sub alien_proc {
 }
 
 1;
+## -----SOURCE FILTER LOG BEGIN-----
+##
+## PerlCritic profile "pcore-script" policy violations:
+## ┌──────┬──────────────────────┬────────────────────────────────────────────────────────────────────────────────────────────────────────────────┐
+## │ Sev. │ Lines                │ Policy                                                                                                         │
+## ╞══════╪══════════════════════╪════════════════════════════════════════════════════════════════════════════════════════════════════════════════╡
+## │    3 │ 118                  │ References::ProhibitDoubleSigils - Double-sigil dereference                                                    │
+## ├──────┼──────────────────────┼────────────────────────────────────────────────────────────────────────────────────────────────────────────────┤
+## │    3 │ 137                  │ ValuesAndExpressions::ProhibitInterpolationOfLiterals - Useless interpolation of literal string                │
+## └──────┴──────────────────────┴────────────────────────────────────────────────────────────────────────────────────────────────────────────────┘
+##
+## -----SOURCE FILTER LOG END-----
 __END__
 =pod
 
 =encoding utf8
 
+=head1 NAME
+
+Pcore::App::Selenium
+
+=head1 SYNOPSIS
+
+=head1 DESCRIPTION
+
+=head1 ATTRIBUTES
+
+=head1 METHODS
+
+=head1 SEE ALSO
+
 =cut
